#!/usr/bin/env python

'''
Simple "Square Detector" program.

Loads several images sequentially and tries to find squares in each image.
'''

# 导入必要的库
from __future__ import print_function
import sys
PY3 = sys.version_info[0] == 3

if PY3:
    xrange = range

import numpy as np
import cv2 as cv
import video


def angle_cos(p0, p1, p2):
    # 计算三个点之间的余弦值
    d1, d2 = (p0-p1).astype('float'), (p2-p1).astype('float')
    return abs( np.dot(d1, d2) / np.sqrt( np.dot(d1, d1)*np.dot(d2, d2) ) )

def find_squares(img):
    # 对图像进行高斯模糊
    img = cv.GaussianBlur(img, (5, 5), 0)
    squares = []
    for gray in cv.split(img):
        for thrs in xrange(0, 255, 26):
            if thrs == 0:
                # 对灰度图像进行Canny边缘检测
                bin = cv.Canny(gray, 0, 50, apertureSize=5)
                bin = cv.dilate(bin, None)
            else:
                # 对灰度图像进行二值化
                _retval, bin = cv.threshold(gray, thrs, 255, cv.THRESH_BINARY)
            # 寻找图像中的轮廓
            contours, _hierarchy = cv.findContours(bin, cv.RETR_LIST, cv.CHAIN_APPROX_SIMPLE)
            for cnt in contours:
                # 对轮廓进行逼近
                cnt_len = cv.arcLength(cnt, True)
                cnt = cv.approxPolyDP(cnt, 0.02*cnt_len, True)
                if len(cnt) == 4 and cv.contourArea(cnt) > 1000 and cv.isContourConvex(cnt):
                    cnt = cnt.reshape(-1, 2)
                    # 计算四个角点之间的最大余弦值
                    max_cos = np.max([angle_cos( cnt[i], cnt[(i+1) % 4], cnt[(i+2) % 4] ) for i in xrange(4)])
                    if max_cos < 0.1:
                        squares.append(cnt)
    return squares

def main():
    # 创建视频捕获对象
    cap = video.create_capture(0)

    # 创建全屏窗口
    cv.namedWindow('squares',cv.WND_PROP_FULLSCREEN)
    cv.setWindowProperty('squares', cv.WND_PROP_FULLSCREEN, cv.WINDOW_FULLSCREEN)


    while True:
        # 读取一帧图像
        _flag, img = cap.read()
        # 在图像中查找方块
        squares = find_squares(img)
        # 绘制找到的方块轮廓
        cv.drawContours( img, squares, -1, (0, 255, 0), 3 )
        # 显示图像
        cv.imshow('squares', img)
        ch = cv.waitKey(1)
        # 按下ESC键退出程序
        if ch == 27:
            break


if __name__ == '__main__':
    print(__doc__)
    main()
    cv.destroyAllWindows()
